function [mo,ro,Co,phio,chainlog,output]=optimum(param,mrange,rrange,flags,modelparam,isim)
%==========================================================================
% Monitoring process for attributes with quality deterioration and 
% diagnosis errors
%
% Anderson Laecio Galindo Trindade
% Linda Lee Ho
% Roberto da Costa Quinino
%
% The aim of this paper is to present on line economical quality control 
% procedure for attributes in a process subject to a quality deterioration 
% after a random shift and misclassification errors during inspections. 
% The process starts in control (State I) and in a random time, it shifts 
% to out of control (State II). Once at State II, the non-conforming 
% fraction increases according to a non-decreasing function psi(z), 
% where z is the number of items produced after a shift. The monitoring 
% procedure consists of inspecting a single item at every m produced items, 
% which is examined r times independently to decide its condition. Once 
% an inspected item declared non-conforming, the process is stopped and 
% adjusted. A direct search technique is used to find the optimal parameters 
% which minimizes the expected cost function.
% 
% version: 2007-03-01 (yyyy-mm-dd)
%
%   Basic Syntax:
%
%   [mo,ro,Co,phio,chainlog]=optimum(param,mrange,rrange,flags,modelparam)
%
%       INPUT:
%       param   vector parameters
%               [pi p1 ci cnc ca cs_nc cs_c alpha beta deltamax phimax]
%               (see paper for definitions)
%       mrange  vector with search limits for m
%               [mmin mmax]
%               Exemples of use:
%               [mmin mmax] = left and right constrained search
%               [] = non constrained search
%       rrange  vector with search limits for r
%               [rmin rmax]
%               Exemples of use:
%               [rmin rmax] = left and right constrained search
%               [] = non constrained search. In this case, the program
%               decides when stops the search, starting by r=1
%       mflags  vector with flags
%               [gflag chainflag sflag]
%               gflag = 1 to show plots, 0 for do not show
%               chainflag = (integer value > 1) defines the chain
%               size to this value. This option overwrite the criterion
%               deltamax. Set chainflag = 0 means to 
%               determine the chain size every time considering deltamax. 
%               Set chainflag = 1 means use, at each step search, the 
%               largest chain size required to match the criterion deltamax
%               considering all history of search.               
%               sflag = 1 stops the search when a minimum is found. sflag =
%               0 stops the search only when match criterion phimax.
%       modelparam  vector parameters used in model of deterioration
%               can be used to easily change parameters of deterioration
%               function. Set modelparam = 1 to use the first model.
%               Set modelparam=2 for linear, modelparam=3 for constant model. 
%               Set modelparam=4 and change file fdet.m to include a user function
%       isim	current simulation in a loop
%       output  matrix with search log. [m,c,r,phi,n]
%
%       OUTPUT:
%       mo          optimum sampling interval
%       ro          optimum number of repetitive classifications
%       Co          optimum cost using mo and ro
%       phio        fraction of defective produced by process considering mo
%                   and ro
%       chainlog    chain size and respective delta to mo, ro and Co   
%       
%
%   See FDET.m
%==========================================================================

% Declaring global variables
global pi p1 ci cnc ca cs_nc cs_c alpha beta alpha0 beta0
global Valfa Vbeta
global MODEL MODELPARAM ISIM
global F

% Converting vetor in individual variables
tic;
[pi,p1,ci,cnc,ca,cs_nc,cs_c,alpha0,beta0,deltamax,phimax]=getdados(param);
phinew=0; Co=inf;

% Allocating space to output
OUTPUT=zeros(10000,5);

vr=1:1:100;
vrmin=floor(0.5*vr);
Valfa=binocdf(vrmin,vr,1-alpha0);
Vbeta=1-binocdf(vrmin,vr,beta0);

%==========================================================================
% Security limits
%==========================================================================
% nmax = maximum chain size
NMAX=100;
% rmax = maximum r
RMAX=100;
% mmax 
MMAX=2e6;

%==========================================================================
% Reading Input values
%==========================================================================

% Validacao
if nargin> 5
    ISIM=isim;
else
    ISIM=1;
end

if nargin > 4
    MODELPARAM=modelparam;
else
    MODELPARAM=1;
end

if nargin > 3
    if length(flags)~=3 
        error('flags needs to be a vector with 3 elements');
    end
    gflag = flags(1);
    chainflag = flags(2);
    if chainflag>NMAX
        chainflag=NMAX;
    end
    sflag = flags(3);
else
    gflag = 0; 
    chainflag = 0;
    sflag = 0;
end

if nargin > 2
    if length(rrange)==2;
        temp=abs(round(rrange));
        rmin=min(temp);
        if max(temp)<RMAX;
            rmax=max(temp);
        else
            rmax=RMAX;
        end
        if rmin<1
            rmin=1;
        end
        if rmax<rmin
            rmax=rmin;
        end
    else
        rmin=1;
        rmax=0;
    end
else
    rmin=1;
    rmax=0;
end


if nargin > 1
    if length(mrange)==2;
        temp=abs(round(mrange));
        mmin=min(temp);
        if max(temp)<MMAX;
            mmax=max(temp);
        else
            mmax=MMAX;
        end
        if mmin<1
            mmin=1;
        end
        if mmax<mmin
            mmax=mmin;
        end
    else
        mmin=1;
        mmax=0;
    end
else
    mmin=1;
    mmax=0;
end

if length(param)~=11 
    error('param needs to be a vector with 11 elements');
end

%==========================================================================
% End of Input values
%==========================================================================
% Pre-calculation fdet
FMAX=100000;
if mmax==0 | (mmax*NMAX) < FMAX
    index=1:1:FMAX;
    F=fdet(index);
else
    index=1:1:max(mmax*5,FMAX);
    F=fdet(index);
end

m=mmin-1;
contm=0;
maxchain=2;

Co=inf;
mo=0;
ro=0;
phio=0;
no=0;

mflag=0;

while phinew<phimax & (m < mmax | mmax==0) & (m<MMAX) & (mflag<5 | sflag==0);
    
    contm=contm+1;
    m=m+1;
    
    rflag=0;    
    r=rmin-1;
    
    Cnew=inf;
    rnew=0;
    
    while (r<rmax | rmax==0) & r<RMAX & rflag<2;

        r=r+1;
        
        if chainflag==0
            n=2;
        elseif chainflag==1
            n=maxchain;
        else
            n=chainflag;
        end

        CHAINLOG=zeros(NMAX-n+1,4);
        contn=1;
        
        Cnold=Cost_mrn(m,r,n);
        delta=2*deltamax;
        
        CHAINLOG(contn,1)=n;
        CHAINLOG(contn,2)=Cnold;
        
        while (delta>deltamax) & (chainflag<2 | contn==1) & (n<=NMAX)
            n=n+1;
            contn=contn+1;
            Cnnew=Cost_mrn(m,r,n);
            delta=abs(Cnnew-Cnold)/Cnnew;
            Cnold=Cnnew;
            
            CHAINLOG(contn,1)=n;
            CHAINLOG(contn,2)=Cnold;
            CHAINLOG(contn,3)=delta;
            if delta<deltamax
                CHAINLOG(contn,4)=1;
            end
        end
        
        if chainflag==1
            maxchain=n-1;
        end       
        
        if Cnnew < Cnew
            Cnew=Cnnew;
            rnew=r;
        end
        rflag=r-rnew;

    end
    
    % Calculating phinew
    alpha=Valfa(rnew);
    beta=Vbeta(rnew);
    [P,y]=prob_model(m,n);
    flagphi=1;
    phinew=(cost_model(m,rnew,n,y,P,flagphi)*y)/m;

    if phinew<phimax
        if Cnew<Co
            Co=Cnew;
            ro=rnew;
            phio=phinew;
            mo=m;
            no=n;
            chainlog=CHAINLOG;
        end
    else
        warning('The statistical criterion was violated.')
    end

    mflag=m-mo;
    
    if rem(contm,5)==0
        status(mo,ro,Co,m,rnew,Cnew,phinew,delta,n)
    end
    
    OUTPUT(contm,1)=m;
    OUTPUT(contm,2)=Cnew;
    OUTPUT(contm,3)=rnew;
    OUTPUT(contm,4)=phinew;
    OUTPUT(contm,5)=n;    
    
end

OUTPUT=OUTPUT(1:contm,:);
output=OUTPUT;

% Calculating chainlog if chainflag>0
if chainflag>0 & no>0
    
    CHAINLOG=zeros(NMAX-n+1,4);
    
    n=2;
    contn=1;
    Cnold=Cost_mrn(mo,ro,n);
    CHAINLOG(contn,1)=n;
    CHAINLOG(contn,2)=Cnold;
    
    for i=3:no
        n=i;
        contn=contn+1;
        Cnnew=Cost_mrn(mo,ro,n);
        delta=abs(Cnnew-Cnold)/Cnnew;
        Cnold=Cnnew;
        
        CHAINLOG(contn,1)=n;
        CHAINLOG(contn,2)=Cnold;
        CHAINLOG(contn,3)=delta;
        
        if delta<deltamax
            CHAINLOG(contn,4)=1;
        end
    end

    chainlog=CHAINLOG;

end

if no>0
    chainlog=chainlog(1:(no-1),:);
else
    chainlog=[];
end

status(mo,ro,Co,m,rnew,Cnew,phinew,delta,n)

if gflag==1
    figure;
    set(gcf,'Color',[1,1,1]);
    
    subplot(2,2,1);
    plot(OUTPUT(:,1),OUTPUT(:,2),'b-',mo,Co,'ro');
    xlabel('m');ylabel('Cost');
    title(['Cost versus m (C^o=' num2str(Co,6) '; r^o=' num2str(ro,6) '; \phi^o=' num2str(phio,6) '; n^o=' num2str(no,6) ')'] );

    subplot(2,2,2);
    plot(OUTPUT(:,1),OUTPUT(:,3),'b-',mo,ro,'ro');
    xlabel('m');ylabel('r');
    title('r versus m');

    subplot(2,2,3);
    plot(OUTPUT(:,1),OUTPUT(:,4),'b-',mo,phio,'ro');
    xlabel('m');ylabel('\phi');
    title('\phi versus m');
            
    subplot(2,2,4);    
    plot(OUTPUT(:,1),OUTPUT(:,5),'b-',mo,no,'ro');
    xlabel('m');ylabel('n');  
    title('n versus m');    
end

function C=Cost_mrn(m,r,n)
%==========================================================================
% Auxiliar Function - Calculates minimum Cost for specific m
%==========================================================================
global pi p1 ci cnc ca cs_nc cs_c alpha beta lambda alpha0 beta0
global Valfa Vbeta

flagphi=0;
alpha=Valfa(r);
beta=Vbeta(r);
[P,y]=prob_model(m,n);
V=cost_model(m,r,n,y,P,flagphi);
C=(V*y)/m;


function [a,b]=alfabetaef(r,rmin,alpha0,beta0)
%==========================================================================
% Auxiliar Function - Calculates alpha* and beta* (after r classifications)
%==========================================================================
a=binocdf(rmin-1,r,1-alpha0);
b=1-binocdf(rmin-1,r,beta0);


function [pi,p1,ci,cnc,ca,cs_nc,cs_c,alpha0,beta0,deltamax,phimax]=getdados(mdados)
%==========================================================================
% Auxiliar Function / Read data from matrix mdados
%==========================================================================
pi=mdados(1);
p1=mdados(2);
ci=mdados(3);
cnc=mdados(4);
ca=mdados(5);
cs_nc=mdados(6);
cs_c=mdados(7);
alpha0=mdados(8);
beta0=mdados(9);
deltamax=mdados(10);
phimax=mdados(11);


function [P,y]=prob_model(m,nw)
%==========================================================================
% Auxiliar Function - Calculates markov chain P and stationary vector Y
%==========================================================================
global p1 p3 alpha beta pi lambda
global F

P=zeros((nw+1)*2,(nw+1)*2);

%P(0,1)(0,1)
P(1,1)=(1-pi)^m*((1-p1)*(1-alpha)+p1*beta);

%P(0,1)(0,2)
P(1,2)=(1-pi)^m*((1-p1)*alpha+p1*(1-beta));

vi=1:1:m;
temp=((1-pi).^(vi-1))*pi;
f=F(m-vi+1);

%P(0,1)(1,1)
P(1,3)=sum(temp.*((1-f)*(1-alpha)+f*beta));

%P(0,1)(1,2)
P(1,4)=sum(temp.*((1-f)*alpha+f*(1-beta)));

% All even rows of the matrix P are equal to the first row
for i=2:2:((nw+1)*2)
    P(i,:)=P(1,:);
end

temp=temp/(1-(1-pi)^m);

for j=2:nw

    i=(j-1)*2+1;
    
    f=F(j*m-vi+1);
    
%P(j-1,1)(j,1)
    P(i,i+2)=sum(temp.*((1-f)*(1-alpha)+f*beta));

%P(j-1,1)(j,2)    
    P(i,i+3)=sum(temp.*((1-f)*alpha+f*(1-beta)));
    
end

% Truncation on size n

% P(n,1)(n,1)
P((nw+1)*2-1,(nw+1)*2-1)=P(nw*2-1,(nw+1)*2-1);

% P(n,1)(n,2)
P((nw+1)*2-1,(nw+1)*2)=P(nw*2-1,(nw+1)*2);


% Getting stationary probabilities 
A=transpose(P)-eye((nw+1)*2);
A((nw+1)*2,:)=ones(1,(nw+1)*2);
B=zeros((nw+1)*2,1);
B((nw+1)*2,1)=1;
y=A\B;


function V=cost_model(m,r,nw,y,P,flagphi)
%==========================================================================
% Auxiliar Function - Calculates cost function
%==========================================================================
global pi p1 ci cnc ca cs_nc cs_c alpha beta
global F

if flagphi==1   %To calculate phi
    oci=ci;
    ocnc=cnc;
    oca=ca;
    ocs_nc=cs_nc;
    ocs_c=cs_c;

    ci=0;
    ca=0;
    cnc=1;
    cs_c=0;
    cs_nc=1;
end

% Initialization
theta=zeros(1,(nw+1)*2);
epsilon=zeros(1,(nw+1)*2);
phi=zeros(1,(nw+1)*2);

%Cost state w0k1
epsilon(1,1)=cnc*(m-1)*p1;
theta(1,1)=cnc*p1*beta/((1-p1)*(1-alpha)+p1*beta);
phi(1,1)=0;

%Cost state w0k2
epsilon(1,2)=cnc*(m-1)*p1;
theta(1,2)=cs_nc*p1*(1-beta)/((1-p1)*alpha+p1*(1-beta))+cs_c*(1-p1)*alpha/((1-p1)*alpha+p1*(1-beta));
phi(1,2)=ca;

%Cost state w1k1
index=1:1:m;
temp1=(pi*(1-pi).^(index-1))/(1-(1-pi)^m);
temp2=(index-1)*p1;

f=F(index);

% Changing for efficiency
% for i=1:m
%     if i<m
%         temp3=1:(m-i);
%         temp2(i)=temp2(i)+sum(f(temp3));
%     end
% end

temp4=temp2*0;
for i=1:1
    if i<m
        temp3=1:(m-i);
        temp4(i)=sum(f(temp3));
        temp2(i)=temp2(i)+temp4(i);
    end
end

for i=2:m
    if i<m
        temp4(i)=temp4(i-1)-f(m-i+1);
        temp2(i)=temp2(i)+temp4(i);
    end
end

% end changing for efficiency

epsilon(1,3)=cnc*sum(temp1.*temp2);

f=F(m-index+1);
temp3=f*beta./((1-f)*(1-alpha)+f*beta);
theta(1,3)=cnc*sum(temp1.*temp3);

phi(1,3)=0;

%Cost state w1k2
epsilon(1,4)=epsilon(1,3);

temp3=f*(1-beta)./((1-f)*alpha+f*(1-beta));
temp4=(1-f)*alpha./((1-f)*alpha+f*(1-beta));
theta(1,4)=cs_nc*sum(temp1.*temp3)+cs_c*sum(temp1.*temp4);

phi(1,4)=ca;


for j=2:(nw)

    % Cost state wjk1    
    
    i=j*2+1;
    temp2=zeros(1,m);
    temp3=1:(m-1);

% Changing for efficiency    
%     for k=1:m
%         if m>1
% 			temp4=m*(j-1)-k+temp3+1;        
% 			f=F(temp4);		
%             temp2(k)=sum(f);
%         end
%     end

    for k=1:1
        if m>1
			temp4=m*(j-1)-k+temp3+1;        
			f=F(temp4);		
            temp2(k)=sum(f);
        end
    end
    
    for k=2:m
        if m>1
            i1=m*(j-1)-(k-1)+(m-1)+1;
            i2=m*(j-1)-(k)+(1)+1;
            temp2(k)=temp2(k-1)-F(i1)+F(i2);
        end
    end
% end changing for efficiency
    
    epsilon(1,i)=cnc*sum(temp1.*temp2);

    f=F(j*m-index+1);
    temp3=f*beta./((1-f)*(1-alpha)+f*beta);
    theta(1,i)=cnc*sum(temp1.*temp3);

    phi(1,i)=0;
    
    % Cost state wjk1  

    i=j*2+2;
    
    epsilon(1,i)=epsilon(1,i-1);
    
    temp3=f*(1-beta)./((1-f)*alpha+f*(1-beta));
    temp4=(1-f)*alpha./((1-f)*alpha+f*(1-beta));    

    theta(1,i)=cs_nc*sum(temp1.*temp3)+cs_c*sum(temp1.*temp4);
    
    phi(1,i)=ca;    

end

%Vetor custo completo
V=(epsilon+theta+phi)+r*ci;


if flagphi==1   %To calculate phi
    ci=oci;
    ca=oca;
    cnc=ocnc;
    cs_c=ocs_c;
    cs_nc=ocs_nc;
end



function status(mo,ro,Co,m,rnew,Cnew,phinew,delta,n);
%==========================================================================
% Auxiliar Function - Display log
%==========================================================================
global ISIM
clc;
fprintf('%60s\n','============================================================');
fprintf('%60s\n','..::                     Search Log                     ::..');
fprintf('%15s\t %13.6f\n','simulac.= ',ISIM);
fprintf('%60s\n','');
fprintf('%15s\t %13.6f\n','m       = ',m);
fprintf('%15s\t %13.6f\n','r_new   = ',rnew);
fprintf('%15s\t %13.6f\n','C       = ',Cnew);
fprintf('%15s\t %13.6f\n','phi     = ',phinew);
fprintf('%15s\t %13.8f\n','delta   = ',delta);
fprintf('%15s\t %13.6f\n','n       = ',n);
fprintf('%60s\n','');
fprintf('%15s\t %13.6f\n','m_o  = ',mo);
fprintf('%15s\t %13.6f\n','r_o  = ',ro);
fprintf('%15s\t %13.6f\n','C_o  = ',Co);
fprintf('%60s\n','============================================================');
